# Calling BAS functions from web interface.

Following API allows to create any number of threads and call BAS functions in that threads. There is possibility to call several functions or one function several time in created thread. This is especially useful if you want to create custom interface from scratch. With the help of this API you can, for example, attach BAS function call to button click and get result as a javascript Promise class.

There are two way of calling BAS functions: by simplified call and by explicitly controlling thread lifetime. Lets reviewing each of them in more details.

## BAS function simplified call.

In order to call BAS function in simple way, you can use RunFunction method:

```javascript
var ResultAsPromise = Api.RunFunction(FunctionName, FunctionParameters)
```

_FunctionName_ is BAS function name as string.

_FunctionParameters_ is arguments list as hash, for example, ```{arg1: "data1", arg2: "data2"}```

_ResultAsPromise_ is always a Promise, it can be used to obtain function result.

_RunFunction_ will return immediately, use _ResultAsPromise_ to wait while function will finish.

Lets review some practical example:

Suppose you have a simple BAS function "LoadUrl", it takes one argument, which is called "Url", and returns page content, which specified url points to. Internally it opens that page in a browser and uses "Page Html" action to obtain result:

<p align="center">
![](//wiki.bablosoft.com/lib/exe/fetch.php?cache=&media=chromeworker_2019-07-30_11-12-13.png ':size=300')
</p>


We will call function "LoadUrl" with parameter "ip.bablosoft.com", like this:

```javascript
Api.RunFunction("LoadUrl", {Url: "ip.bablosoft.com"})
```

After following code will be executed, BAS will create a new thread, then run a "LoadUrl" function there, after function will finish, thread will be removed automatically. 

## Obtaining results.

_RunFunction_ method will return object of Promise type, not a function result. Also note, that Promise will be returned immediately. So if we want to obtain page content as a string, or wait while function will finish, you need to use either _then_ method, or _await_ keyword:

```javascript
Api.RunFunction("LoadUrl", {Url: "ip.bablosoft.com"})
	.then(function(Result){
		console.log("Page content is " + Result)
	})
```

or

```javascript
var Result = await Api.RunFunction("LoadUrl", {Url: "ip.bablosoft.com"})
console.log("Page content is " + Result)
```

Also remember, that _await_ is possible to use only inside functions marked as _async_, so if, for example, you attach function call to button click handler, you should not forget to add _async_ to function declaration:

```javascript
$("#Start").click(async function(){
	var Result = await Api.RunFunction("LoadUrl", {Url: "ip.bablosoft.com"})
	//Do something with Result
})
```

## Handling errors.

What if specified url doesn't exist? In that case BAS function will fail, and you can process that error with a help of _catch_ method in case if you are using Promise or with try/catch keywords:

```javascript
Api.RunFunction("LoadUrl", {Url: "wronurl.com"})
	.then(function(Result){
		console.log("Page content is " + Result)
	})
	.catch(function(Message){
		console.log("Function finished with error " + Message)
	})
```

or

```javascript
try
{
	var Result = await Api.RunFunction("LoadUrl", {Url: "ip.bablosoft.com"})
	console.log("Page content is " + Result)
}catch(e)
{
	console.log("Function finished with error " + e)
}
```

## Stopping function execution.

Normally, Promise object won't let you interrupt its execution, but the Promise object returned as result of calling BAS function has a _stop_ method. This method will immediately interrupt function execution and remove thread which runs it. Here is a simple usage example, which will stop function after running it for 10 seconds:

```javascript
var ResultAsPromise = Api.RunFunction("LongRunningFunction", {})
setTimeout(function(){
	//Stop function immediately
	ResultAsPromise.stop()
},10000)
	
```

After function is interrupted, corresponding Promise object will throw with message "Task stopped manually".

A good practice would be to save Promise object and stop it on a timeout, button click, or some other event later.

```javascript
var Task = null

$("#StartTask").click(function(){
	Task = Api.RunFunction("LongRunningFunction", {})	
})

$("#StopTask").click(function(){
	//Check if task has been started, if so, stop it.
	if(Task)
		Task.stop()
})

```

## Managing thread lifetime manually.

What will happen if we use _RunFunction_ several times?

```javascript
Api.RunFunction("Function1", {})
Api.RunFunction("Function2", {})
Api.RunFunction("Function3", {})
```

This code will create 3 threads, run Function1 in first thread, Function2 in second thread, etc. in order to run all 3 functions in one thread, you need to use _BASThread_ class, it has following methods:

```javascript
var Thread = new BASThread()

//This method is pretty same as Api.RunFunction, except
//it runs function in specified thread.
var ResultAsPromise = Thread.RunFunction(FunctionName, FunctionParameters)

//Check if thread is already busy with running function
var IsRunningFunction = Thread.IsRunningFunction()

//Stop thread immediately
Thread.StopThread()
```

Now lets rewrite our code, so all 3 functions will start in one thread sequentially.

```javascript
//Creating a new BAS thread
var Thread = new BASThread()

var RunSequence = Promise.resolve()

RunSequence.then(function(){
	return Thread.RunFunction("Function1", {})
}).then(function(){
	return Thread.RunFunction("Function2", {})
}).then(function(){
	return Thread.RunFunction("Function3", {})
}).then(function(){
	Thread.StopThread()
})

```

Now each BAS function will run in one thread and finally thread will be removed. However, this code is not very convenient, more clearer way would be to use await keyword:

```javascript
//Creating a new BAS thread
var Thread = new BASThread()

await Thread.RunFunction("Function1", {})
await Thread.RunFunction("Function2", {})
await Thread.RunFunction("Function3", {})
Thread.StopThread()
```

More complex example:

```javascript
//Creating a new BAS thread
var VisitorThread = new BASThread()

//Visit all sites from list in a loop
var Sites = ["google.com", "gmail.com", "fb.com"]
for(var i = 0; i < Sites.length; i++)
{
	var CurrentUrl = Sites[i]
	await VisitorThread.RunFunction("LoadUrl", {Url: CurrentUrl})
}

Thread.StopThread()
```

You don't have to call next function immediately after previous was finished, you can do that at any time, for example on button click:

```javascript
var Thread = new BASThread()

$("#StartTask1").click(function(){
	Thread.RunFunction("Task1", {})	
})

$("#StartTask2").click(function(){
	Thread.RunFunction("Task2", {})	
})

$("#StartTask3").click(function(){
	Thread.RunFunction("Task3", {})	
})

```

Several functions are not allowed to run simultaneously in one thread. In case if you try to do that, exception "Other task is executing, can't start new task one" will be thrown. In order to check, if thread is busy at the moment, use _IsRunningFunction_ method: 

```javascript
var Thread = new BASThread()

$("#StartTask1").click(function(){
	if(!Thread.IsRunningFunction())
		Thread.RunFunction("Task1", {})	
})

$("#StartTask2").click(function(){
	if(!Thread.IsRunningFunction())
		Thread.RunFunction("Task2", {})	
})

$("#StartTask3").click(function(){
	if(!Thread.IsRunningFunction())
		Thread.RunFunction("Task3", {})	
})

```

_StopThread_ method will stop whole thread, not just current task. For example, closing thread will force browser to close. In case if you call _RunFunction_ method on closed thread, a new thread will be created again:

```javascript
var Thread = new BASThread()

//url1.com and url2.com will be visited in one browser
await Thread.RunFunction("LoadUrl", {Url: "url1.com"})
await Thread.RunFunction("LoadUrl", {Url: "url2.com"})

//First browser will be closed
Thread.StopThread()

//url1.com and url2.com will be visited with another browser again
await Thread.RunFunction("LoadUrl", {Url: "url1.com"})
await Thread.RunFunction("LoadUrl", {Url: "url2.com"})

//Second browser will be closed
Thread.StopThread()

```


## Debugging BAS function call.

Debugging function call in record mode is quite straightforward. You can execute action one by one and see how results are reflected on web interface:

<p align="center">
![](//wiki.bablosoft.com/lib/exe/fetch.php?cache=&media=2019-07-30_13-45-27.gif ':size=300')
</p>

Debugging function call in interface editor is quite trickier, because interface editor has no connection to BAS and therefore can't execute BAS functions. If you try to call function from interface editor, you will get a random string as result in a several seconds. This behavior is not acceptable in several cases: if your function return a list, not a string, or string in specific format, if you want to test error handling, etc. In order to circumvent this limitation. you need to define "EmulateFunctionRun" function inside web interface, it has following specification:

```javascript
//Define function, which will imitate BAS function call
function EmulateFunctionRun(FunctionName, FunctionParameters, Resolve, Reject)
{
	//...
}
```

_FunctionName_ - is a string, which contains BAS function name, can be used to return different results, depending on which function is called.


_FunctionParameters_ - is arguments list as hash, for example, ```{arg1: "data1", arg2: "data2"}```

_Resolve_ - is a function, which will be called to return successful result.

_Reject_ - is a function, which will be called to return failure message.

Here is default implementation:


```javascript
function EmulateFunctionRun(FunctionName, FunctionParameters, Resolve, Reject)
{
	//Generate random data
	var RandomTime = Math.floor(Math.random() * (5000 - 3000) ) + 3000
	var RandomResult = Math.floor(Math.random() * (1000))

	//Return result in RandomTime milliseconds
	setTimeout(function(){
		//Generate result string
		Resolve("Test result " + RandomResult)
	},RandomTime)

}
```

If you have function "CheckIp", which requires string with ip address, you can modify _EmulateFunctionRun_ like this:

```javascript
function EmulateFunctionRun(FunctionName, FunctionParameters, Resolve, Reject)
{
	//Custom result for CheckIp function
	if(FunctionName == "CheckIp")
	{
		Resolve("127.0.0.1")
		return
	}

	//Generate random data
	var RandomTime = Math.floor(Math.random() * (5000 - 3000) ) + 3000
	var RandomResult = Math.floor(Math.random() * (1000))

	//Return result in RandomTime milliseconds
	setTimeout(function(){
		//Generate result string
		Resolve("Test result " + RandomResult)
	},RandomTime)

}
```

This will help to test web interface much better even without connection to BAS.


## Function statistic.

In order to enumerate all running functions, you can use following method:

```javascript
var RunningTasks = Api.GetTasks()
```

_GetTasks_ method will return associative array, which has task ids as keys and object representing task as values. Lets review its content by example.
```javascript
{
	//Task with id 1
	1:
	{
		"task_id": 1, 
		"type": "function",
		"browser_id": 3,
		"function_name": "Test",
		"object": Promise, 
		"parameters": 
		{
			"param1": "value1"
		},
		"started_at": 1568894085100
	},

	//Task with id 2
	2:
	{
		"task_id": 2, 
		"type": "thread",
		"browser_id": 4,
		"function_name": "Test",
		"object": BASThread, 
		"parameters": 
		{
			"param1": "value2"
		},
		"started_at": 1568894085100
	}
}
```

_task&#95;id_ is task identifier, it is generated randomly when task is started.

_type_ is a string which identifies task type, it could be either "function" if you started task with ```Api.RunFunction``` method or "thread" if you created a thread with ```new BASThread()```.

_browser&#95;id_ is unique browser identifier, can be used to, for example, show or hide browser, see [inspecting browser](inspectingbrowser.md) section.

_function&#95;name_ is a string which contains BAS function name.

_object_ is a javascript object, depending on _type_ is will have Promise or BASThread type. You can use that object to stop task instantly, see "Stopping function execution" section.

_parameters_ is a arguments list as hash. It contains parameters, with which task was started.

_started&#95;at_ is a timestamp, when task started.

Api.GetTasks can be used at any point of time, it will return data which reflect actual script state.

**Example #1.** Stop all tasks:

```javascript
var RunningTasks = Api.GetTasks()

//Iterate on all running tasks
Object.values(RunningTasks).forEach(function(Task){
	
	//Stop thread or promise depending on type
	if(Task.type == "function")
		Task.object.stop();
	else
		Task.object.StopThread();
})
```

**Example #2.** Stop only long running tasks:

```javascript
var RunningTasks = Api.GetTasks()
Object.values(RunningTasks).forEach(function(Task){
	
	//Stop only task running for at least hour
	if(Task.started_at < Date.now() - 1000 * 60 * 60)
	{
		if(Task.type == "function")
			Task.object.stop();
		else
			Task.object.StopThread();
	}
})
```


**Example #3.** Show browser for specific thread:

```javascript
var ThreadId = 1

//Get task to find browser id
var Task = Api.GetTasks()[ThreadId]

//Check if task with specified id exists
if(Task)
{
	Api.ShowBrowser(Task.browser_id)
}
```

In order to obtain task id from a thread object, you can use ```GetId``` or ```get_id``` methods. GetId works only with BASThread object and get_id with Promise.

```javascript
//Start thread and run function
var Thread = new BASThread()
var Wait = Thread.RunFunction("Test", {})

//Get thread id
var ThreadId = Thread.GetId()

//Wait for results and stop thread
var Result = await Wait
Thread.StopThread() 
```


or


```javascript
//Run function
var Wait = Api.RunFunction("Test", {})

//Get thread id
var ThreadId = Wait.get_id()

//Wait for results
var Result = await Wait
```
