# Filesystem API

Unlike real browser, browser which runs web interface has set of method to access file system. All these methods are synchronous.


## Method _ReadFile_


Read file into variable, treat file content as a string with specified encoding. Encoding can be omitted.

```javascript
let FileContent = ReadFile(Path, Encoding)
let FileContent = ReadFile(Path)
```

Usage example:

```javascript
//Read file with UTF-8 encoding
let FileContent = ReadFile("c:/data/names.txt")

//Read file with Windows-1251 encoding
let FileContent = ReadFile("c:/data/names.txt", "Windows-1251")
```

By default following method will recognize automatically files with UTF-8 with BOM, UTF-8 without BOM, UTF-16LE with BOM, UTF-16BE with BOM encodings, all other must be specified explicitly.

In case if file doesn't exist or can't be read, this method will throw error:

```javascript
try
{
	let FileContent = ReadFile("c:/data/names.txt")
}catch(e)
{
	console.log("Error during file read " + e)
}
```

Reading file can be combined with [OpenFileDialog](/interactingnativewindow?id=method-openfiledialog) in order to let user choose file path:

```javascript
Api.OpenFileDialog({}).then(function(SelectedFile){
    console.log(ReadFile(SelectedFile))
})
```

Finally, here is a useful snippet to read file as lines:

```javascript
let LinesList = ReadFile("c:/data/names.txt").match(/[^\r\n]+/g);
```

## Method _ReadFileAsBinary_

Following method is almost the same as [ReadFile](/interactingnativewindow?id=method-readfile), but instead of returning string, it returns result with Uint8Array type, it also doesn't require encoding for obvious reasons.

```javascript
let FileData = ReadFileAsBinary(Path)
```

This method is perfect for reading binary data. 

```javascript
ReadFileAsBinary("c:/data/test.txt")
//If test.txt contains 123 string, result will be Uint8Array(3) [49, 50, 51]
```

In case if file doesn't exist or can't be read, this method will throw error.


## Method _WriteFile_

Overwrite existing file with specified data or create new file. This method can have three different modes:


```javascript
WriteFile(Path, DataAsString)

//Write string into file, original file content will be removed
WriteFile("c:/data/test.txt", "test")
```

```javascript
WriteFile(Path, DataAsString, Encoding)

//Write string into file with specified encoding.
WriteFile("c:/data/test.txt", "test", "UTF-16")
```

```javascript
WriteFile(Path, RawData)

//Write raw data into file, data must have Uint8Array type
WriteFile("c:/data/test.txt", new Uint8Array([49, 50, 51]))
```

If parent folder doesn't exists, it will be created.

If file can't be changed or data failed to write, this method will throw an error.


## Method _AppendFile_

Works absolutely the same as [WriteFile](/filesystem?id=method-writefile), but instead of overwriting file appends data to the file end. 

```javascript
AppendFile(Path, DataAsString)

//Append string to file end, create new one if not exist
AppendFile("c:/data/test.txt", "test")
```

```javascript
AppendFile(Path, DataAsString, Encoding)

//Append string to file with specified encoding.
AppendFile("c:/data/test.txt", "test", "UTF-16")
```

```javascript
AppendFile(Path, RawData)

//Append raw data to file end, data must have Uint8Array type
AppendFile("c:/data/test.txt", new Uint8Array([49, 50, 51]))
```

This method is very useful for writing logs, here is how new line can be appended:

```javascript
AppendFile("c:/data/test.txt", LogMessage + "\r\n")
```

If parent folder doesn't exists, it will be created.

If file can't be changed or data failed to write, this method will throw an error.


## Method _FileExists_

Check if file or directory with specified path exists. This function will never fail and never throw an exception.


```javascript
let IfFileExists = FileExists(Path)
```

## Method _FileSize_

Gets file size in bytes. This function will never fail and never throw an exception. If path points to directory or it is invalid, this function return zero.


```javascript
let FileSize = FileSize(Path)
```


## Method _FileLastWriteTime_

Returns the time of last data modification of file. Return value has number type holding the number of seconds since 00:00, Jan 1 1970 UTC. This function will never fail and never throw an exception. If path points to directory or it is invalid, this function return zero.

```javascript
let FileSize = FileSize(Path) //1590158184 - this value corresponds to Fri May 22 2020 17:36:52
```


## Method _ParentFolder_

Gets parent folder for specified path. This function will never fail and never throw an exception. It also works for non existing files or folders.


```javascript
let ParentPath = ParentFolder(Path)

//Example:
ParentFolder("c:/data/test.txt") // c:/data
ParentFolder("c:/data") // c:/

```

## Method _IsFolder_

Check if path is a valid existing directory. This function will never fail and never throw an exception.


```javascript
let PathIsFolder = IsFolder(Path)
```

## Method _CreateFolder_

Creates folder with specified path.

```javascript
CreateFolder(Path)
```

If folder can't be created, this method will throw an error. If directory is already exists, this function will do nothing.


## Method _RemoveFileOrFolder_

Removes file if specified path points to file, or removes folder recursively if path point to folder.

In case if path is invalid, does nothing.

If file or folder can't be removed, throws exception.

```javascript
RemoveFileOrFolder(Path)

//Examples
RemoveFileOrFolder("c:/data/") //Recursively deletes files in folder and folder itself
RemoveFileOrFolder("c:/data/not exists") //Does nothing
RemoveFileOrFolder("c:/data/locakedfile.txt") //Throw exception
```

## Method _MoveFileOrFolder_

Moves file if specified path points to file, or folder recursively if path point to folder.

Throws exception if task can't be accomplished.

Destination path will be completely removed and then replaced with source path, all files will be overwritten.

```javascript
MoveFileOrFolder(SourcePath, DestinationPath)

//Examples
MoveFileOrFolder("c:/data1", "c:/data2") //rename data1 folder to data2
MoveFileOrFolder("c:/data/file1.txt","c:/file1.txt") //Move file1.txt to c:/file1.txt path
```

## Method _CopyFileOrFolder_

Copy file if specified path points to file, or folder recursively if path point to folder.

Throws exception if task can't be accomplished.

Destination path will be completely removed and then replaced with source path, all files will be overwritten.

```javascript
CopyFileOrFolder(SourcePath, DestinationPath)

//Examples
CopyFileOrFolder("c:/data1", "c:/data2") //copy data1 folder to data2
CopyFileOrFolder("c:/data/file1.txt","c:/file1.txt") //Copy file1.txt to c:/file1.txt path
```

## Method _FolderContent_

Gets list with all files and folders in specified path.

Throws exception if task can't be accomplished.

```javascript
let Items = FolderContent(Path)

//Example:
FolderContent("c:/data")
/* 
	[
		"c:/data/folder1",
		"c:/data/folder2",
		"c:/data/file1.txt",
		"c:/data/file2.txt",
	]
*/
```
This method doesn't distinguish, if result item list is folder or file, to do that, use [IsFolder](/filesystem?id=method-isfolder) method.