# Interacting with database

Custom interface API has comprehensive sets of methods to work with database.



## Method _GetDatabaseStructure_

```javascript
var Structure = Api.GetDatabaseStructure()
```

This is very important method. Id returns list of ids for tables, columns, groups. Ids are used to interact with those entities. For example, to delete group, you need to know its id, and this method is the only way to get it.

This method works fast and returns immediately.

Here is return data example:

```javascript
[
  {
    "columns": [
      {
        "description": "c1",
        "id": 2,
        "name": "c1",
        "type": "string"
      },
      {
        "description": "c2",
        "id": 3,
        "name": "c2",
        "type": "int"
      }
    ],
    "description": "table1",
    "groups": [
      {
        "description": "All data in table",
        "id": "-1",
        "name": "All"
      },
      {
        "description": "Items, which is not present in any group",
        "id": "-2",
        "name": "Not in groups"
      }
    ],
    "id": 1,
    "name": "table1"
  }
]
```

This method call result is array with a list of tables, each table object has id, list of groups and list of columns. You can find several snippets below. Obtaining first table id:

```javascript
Api.GetDatabaseStructure()[0].id
```

Obtaining table id by name:

```javascript
Api.GetDatabaseStructure()
   .find(function(table){return table.name == "table1"})
   .id
```

Use same technique to find columns and groups.



## Event _database&#95;structure&#95;changed_

```javascript
if (EventType == "database_structure_changed") {
   //Code
}
```

This event is fired every time database structure is changed. BAS can change columns and tables structure only after restart, but groups can be modified during script run, so this event will be fired each time group id is changed. You can be sure that column ids and names and tables won't be changed during script run. Changing records won't trigger that event.





## Method _ShowDatadaseManager_

```javascript
Api.ShowDatadaseManager()
```


Show database manager:

<p align="center">
![](//wiki.bablosoft.com/lib/exe/fetch.php?cache=&media=databasemanager2.png ':size=300')
</p>



## Method _DatabaseSelect_

```javascript
Api.DatabaseSelect(Selector, TableId).then(function(RecordList){
   //Code
})
```

This method executes database query and returns result as a list of records. It returns Promise, use _then_ keyword to get result. Second parameter is a table identifier, could be obtained with _GetDatabaseStructure_ method. First parameter is a complex object which specifies database query. Here are several examples.

Select all records from database:

```javascript
{}
```

Sort records by column with id 2 in ascending order:

```javascript
{Sort:{ColumnId: 2, OrderIsAsc: true}}
```

Get first page with page size 10:

```javascript
{Page:{PageNumber: 1, PageSize: 10}}
```

Get records from group with id _5b582547cee1c8c66a85ddd9_:

```javascript
{Groups:{GroupIdList:["5b582547cee1c8c66a85ddd9"]}}
```


Get records from first page of group with id _5b582547cee1c8c66a85ddd9_:

```javascript
{Page:{PageNumber: 1, PageSize: 10}, Groups:{GroupIdList:["5b582547cee1c8c66a85ddd9"]}}
```

Select only records, which column with id 2 contains string _test_:

```javascript
{
   Filter:
   {
      FilterList:
      [
         {
            ColumnId:2,
            Type:"StringContains",
            Data:"test",
         }
      ]
   }
}
```

Filter may have several types depending on column type:

Filters for strings: StringNotEmpty, StringEmpty, StringContains

Filters for numbers: IntEquals, IntNotEquals, IntGreaterOrEqual, IntGreater, IntLessThan, IntLessThanOrEqual

Filters for booleans: BoolIsTrue, BoolIsFalse

Filters for dates: DateIsLessThan, DateGreaterThan, DateIsSameYear, DateIsSameMonth, DateIsSameDay, DateIsSameHour

Method _DatabaseSelect_ returns list with records. Each record is represented with object. Object contains _id_ string and _data_ object with keys equals to column ids and values which equals to actual values for that record. Here is example of returning list:

```javascript
[
  {
    "data": {
      "2": "test1",
      "3": 1
    },
    "id": "5b5820d2cee1c8c66a85ddd7"
  },
  {
    "data": {
      "2": "test2",
      "3": 2
    },
    "id": "5b5820d8cee1c8c66a85ddd8"
  }
]
```




## Method _DatabaseInsert_

```javascript
Api.DatabaseInsert(Groups, Item, TableId).then(function(InsertedRecordId){
   //Code
})
```

This method inserts single record into database.

First parameter is group id list, leave it empty if you don't want to assign groups to new record.

Second parameter is an item object with keys equals to column ids and values which equals to actual values for that record.

Third parameter is table id.

Example of usages:

```javascript
Api.DatabaseInsert(
   [],
   {
      2: "test3",
      3: 3
   }, 
   1)
```




## Method _DatabaseCount_

```javascript
Api.DatabaseCount(Selector, TableId).then(function(RecordCount){
   //Code
})
```

Count number of records matching selector. This method returns Promise, use _then_ keyword to get result. Selector format is the same as in _DatabaseSelect_ method. 






## Method _DatabaseUpdate_

```javascript
Api.DatabaseUpdate(ItemId, Item, TableId).then(function(RecordCount){
   //Code
})
```

Update existing record.

First parameter must equal to record id, which will be changed.

Second parameter is item object, it has same format as second parameter of _DatabaseInsert_ method.

Third parameter is table id.





## Method _DatabaseDelete_

```javascript
Api.DatabaseDelete(Items, TableId)
```

Delete one or several records.

First parameter must equal to list with record ids, which will be deleted.

Second parameter is table id.

Example:

```javascript
Api.DatabaseDelete(["5b5820d8cee1c8c66a85ddd8"], 1)
```






## Method _DatabaseInsertGroup_

```javascript
Api.DatabaseInsertGroup(Group, TableId)
```

Insert group.

Example:

```javascript
Api.DatabaseInsertGroup({Name:"GroupName",Description:"Description"}, 1).then(function(InsertedGroupId){
   //Code
})
```






## Method _DatabaseDeleteGroupNoData_

```javascript
Api.DatabaseDeleteGroupNoData(GroupId, TableId)
```

Delete group, don't delete data inside it.






## Method _DatabaseDeleteGroupWithData_

```javascript
Api.DatabaseDeleteGroupWithData(GroupId, TableId)
```

Delete group and all its records.





## Method _DatabaseUpdateGroup_

```javascript
Api.DatabaseUpdateGroup(GroupId, Group, TableId)
```

Updated group name and description.

Example:

```javascript
Api.DatabaseUpdateGroup("5b5820d8cee1c8c66a85ddd8",{Name:"NewName",Description:"NewDescription"}, 1)
```





## Method _DatabaseMoveToGroup_

```javascript
Api.DatabaseMoveToGroup(Items, GroupId, TableId)
```

Move records to group. Records are identified by first parameter, this parameter must hold a list with record ids.

Second parameter must contain group id.






## Method _DatabaseCopyToGroup_

```javascript
Api.DatabaseCopyToGroup(Items, GroupId, TableId)
```

Copy records to group. Records are identified by first parameter, this parameter must hold a list with record ids.

Second parameter must contain group id.





