# Managing script lifetime

By default interface has 3 states: _initial_, _running_, _stopped_. Initial state requires user to input resources and hit _Run_ button, after that script state is changed to running. User may stop script and it sets script state to _stopped_, or restart it, which restores state to _initial_ again.
_Stopped_ script state is almost same as _running_, except that interface is not updated anymore, but user can view it in readonly mode. This is how BAS script works by default, but you can change that with a help of custom interface and API. For example, you can input resources instead of user and run the script immediately after it starts. Or you can skip _stopped_ state. It is impossible to run several scripts simultaneously with custom interface, but you can achieve same result by splitting functionality inside different threads.


## Event _start_

```javascript
if (EventType == "start") {
   //Code   
}
```

Event _start_ is fired when interface state is changed to running. Custom interface may or may not show log panel, browser view, etc.


## Event _stop_

```javascript
if (EventType == "stop") {
   //Code
}
```

Event _stop_ is fired when interface state is changed to stopped. It could happen because user stopped it manually, or for some other reason, in any case this event will be fired. Custom interface may show report or disable log or react in any other way.



## Event _restart_

```javascript
if (EventType == "restart") {
   //Code
}
```

Script is restarted, it means that resource selection dialog must be shown and user must input all required values again. Interface state is changed to _initial_. Custom interface should return to the same state as it was in the beginning.

<p align="center">
![](//wiki.bablosoft.com/lib/exe/fetch.php?cache=&media=scriptstates.gif ':size=300')
</p>


## Method _AcceptResources_

```javascript
Api.AcceptResources(IsEmptyProject)
```

This method changes script state from _initial_ to _running_. If script state is not _initial_, this method will do nothing. If IsEmptyProject parameter is set to true, then "Main" function will be replaced with infinite sleep and thread number will be set to 1. This option is very usefull if you don't want to use autogenerated interface and want to run tasks manually. After calling this method, BAS will try to get values for all resources: get filenames selected by user, strings, numbers. This process is also customizable. To obtain resource value BAS calls function _GetResourceValue_ with parameter which equals resource name, like this:
```javascript
GetResourceValue("username")
```

_GetResourceValue_ must return a string with resource value. This function must be defined and implemented by custom interface. This is very powerful mechanism, which gives full control over script inputs. Lets look at several possible implementations. For example, if script has 2 resources - username and password, _GetResourceValue_ may look like this.
```javascript
function GetResourceValue(ResourceName)
{
   if(ResourceName == "username")
   {
      //This means, that username is always equal to 'login1'
      return "login1"
   }
   if(ResourceName == "password")
   {
      //This means, that password is always equal to 'pass1'
      return "pass1"
   }
   //Unknown resource name
   return ""
}
```

Previous implementation makes resource values static and always equals to same strings. Following approach may be used if you want to skip initial script state and proceed to running state immediately. You may use javascript or jquery to get user inputs:

```javascript
function GetResourceValue(ResourceName)
{
   if(ResourceName == "username")
   {
      //Return value of input with id "username"
      return $("#username").val()
   }
   if(ResourceName == "password")
   {
      //Return value of input with id "password"
      return $("#password").val()
   }
   //Unknown resource name
   return ""
}
```

Default implementation looks like this:

```javascript
function GetResourceValue(ResourceName) {
    return GetValue(ResourceName)
}
```

By default _GetResourceValue_ will get actual user inputs by using _GetValue_ function. You can take a default implementation as a foundation and tweak it, for example: 

```javascript
function GetResourceValue(ResourceName) {
    if(ResourceName == "username")
    {
       //This means, that username is always equal to 'login1'
       return "login1"
    }
    //Default implementation for everything else
    return GetValue(ResourceName)
}
```

This is a mix of dynamic and static implementation, if _username_ resource is queried, then static string is returned, else function returns actual user input.


_SetResourceValue_ may be or may not be defined. This function is called every time script gets started and for each resource. If you have a custom controls in your interface and want to preserve values across script executions, you need to define, how theese controlls will be filled with specified values. It works pretty similiar to _GetResourceValue_. Example:
```javascript
function SetResourceValue(ResourceName, ResourceValue)
{
    if(ResourceName == "username")
    {
       //Or fill resource in any other way
       $("#Username").val(ResourceValue)
       return
    }
   //Default implementation
   SetValue(ResourceName, ResourceValue)
}
```


## Method _Restart_

```javascript
Api.Restart()
```

This method changes script state from running to initial. If script state is not running, this method will do nothing.



## Method _Stop_

```javascript
Api.Stop(true)
```

This method changes script state from _running_ to _stopped_. If script state is not _running_, this method will do nothing. This method has one parameter - _IsInstant_. If it equals to true, then script execution will be terminated immediately, otherwise BAS will wait until all thread will end and all data will be proceeded. After script will be terminated, custom interface will receive _stop_ event.


